
package com.dxhy.manage.service;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.IService;
import com.dxhy.common.utils.PageUtils;
import com.dxhy.manage.entity.SysDeptEntity;
import com.dxhy.manage.model.BlocDataModel;

/**
 * 系统配置信息
 *
 * @author jiaohongyang
 */
public interface SellersStatService extends IService<SysDeptEntity> {
    /**
     * 查询
     *
     * @param userId
     *            userId
     * @return 返回
     */
    List<SysDeptEntity> queryList(String userId);

    /**
     * 查询
     *
     * @param s
     *            入参
     * @return 返回
     */
    List<SysDeptEntity> queryTaxnoList(String s);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryTotal(Map<String, String> params);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @param response
     *            response
     */
    void exportExcel(Map<String, String> params, HttpServletResponse response);

    /**
     * 表头
     *
     * @return 返回
     */
    JSONObject getTitle();

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryYear(Map<String, String> params);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryMonth(Map<String, String> params);

    /**
     * 表头
     *
     * @return 返回
     */
    JSONObject getYearTitle();

    /**
     * 表头
     *
     * @return 返回
     */
    JSONObject getMonthTitle();

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryConfirmTotal(Map<String, String> params);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryResultPage(Map<String, String> params);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryYearPage(Map<String, String> params);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryMonthPage(Map<String, String> params);

    /**
     * 查询
     *
     * @param params
     *            入参
     * @return 返回
     */
    PageUtils queryConfirmTotalPage(Map<String, String> params);

    BlocDataModel getBlocData(SysDeptEntity sysDeptEntity, Map<String, String> params);

    JSONObject getBlocTitle();

}
